/*
 *  solver.cpp
 * Implements a Newton's Method Solver for a specific system of two 
 * non-linear equations
 *
 * Copyright 2008 Aaswath Raman.
 * The author grants permission to copy, distribute and display this 
 * work in unaltered form, with attribution to the author, for 
 * noncommercial purposes only. All other rights, including commercial 
 * rights, are reserved to the author.
 *  
 *  ----------
 *
 *     Compile:     g++ solver.cpp -o solver
 *     Run:         ./solver
 *
 *     ... Assumes existence of file "./input_data.txt" and generates
 *         from it the file "./results.txt".
 */
 
#include <math.h>
#include <iostream.h>
#include <fstream.h>

void solve(double *x, double k_xn, double alpha);
 
int main()
 {
    double *x, k_xn;
    ifstream fin;     
	ofstream fout;
	
	x = new double[2];
	fout.open("results.txt");
	fout << "k_xn alpha k_x	k_y k_m" << endl; 
	
	for (double alpha=0.6;alpha <=1.0; alpha=alpha+0.2) {
		fin.open("input_data.txt",ios::in);    
		
		fin >> k_xn;
		while (!fin.eof())
		{
			// initial guesses
			x[0] = (k_xn+1)/2;
			x[1] = (k_xn+1)/2;
			solve(x,k_xn,alpha);
			cout << "Running... for alpha, " << alpha << ", k_xn value: " << k_xn << endl;
			fout << k_xn << "	" << alpha << "	" << x[0] << "	" << x[1] << "	" << (x[0]+x[1])/2 << endl;
		
			// get next k_xn value
			fin >> k_xn;
		}
		fin.close();
		fout << endl << "============================================" << endl << endl;
	}
	
	fout.close();
    
    delete x;
    return 0;
}


void solve(double *x,double xn, double alpha)
{
    double **jcbn,*rp,*r,*x_new,*holder,curr,temp;
	
	r = new double[2];
    rp = new double[2];
    x_new = new double[2];
	jcbn = new double *[2];
    jcbn[0] = new double[2];
    jcbn[1] = new double[2];
    	
	// in case of no convergence, time out after 1000 loops
	for (int i=0; i<1000; i++) {
	
		// compare against old, update x, test for convergence and break 
		if (i >= 1) {
			curr = (fabs(x[1]-x_new[1]) + fabs(x[0]-x_new[0]))/2;
			x[0] = x_new[0];
			x[1] = x_new[1];
			if (curr <= 1e-5) {
				delete r; delete rp; delete x_new; delete jcbn;
				return;
			}
		}
		
		// 'current' result (ie: eqn=0)
		r[0] = 2*(x[0]-xn) + 2*alpha*x[1]*(x[0]*x[1]-1);
		r[1] = 2*(x[1]-1) + 2*alpha*x[0]*(x[0]*x[1]-1);
		        
		// let's calculate the jacobian
        curr = x[0];
        x[0] += 1e-12;
		rp[0] = 2*(x[0]-xn) + 2*alpha*x[1]*(x[0]*x[1]-1);
		rp[1] = 2*(x[1]-1) + 2*alpha*x[0]*(x[0]*x[1]-1);
        x[0] = curr;       
        jcbn[0][0] = (rp[0]-r[0])/1e-12;
        jcbn[1][0] = (rp[1]-r[1])/1e-12;
			
		curr = x[1];
		x[1] += 1e-12;
		rp[0] = 2*(x[0]-xn) + 2*alpha*x[1]*(x[0]*x[1]-1);
		rp[1] = 2*(x[1]-1) + 2*alpha*x[0]*(x[0]*x[1]-1);
        x[1] = curr;       
        jcbn[0][1] = (rp[0]-r[0])/1e-12;
        jcbn[1][1] = (rp[1]-r[1])/1e-12;
        		
		//new result vector
		rp[0] = x[0]*jcbn[0][0] + jcbn[0][1]*x[1] - r[0];
		rp[1] = x[0]*jcbn[1][0] + jcbn[1][1]*x[1] - r[1];
				
		// swap jacobian rows (and result vectors), if necessary, for gaussian elimination
		if (jcbn[0][0] == 0) {
			holder = jcbn[1];
			jcbn[1] = jcbn[0];
			jcbn[0] = holder;
			temp = rp[0];
			rp[1] = rp[0];
			rp[0] = temp;			
		}
		
		// gauss/jordan elimination to calculate new variable (x) values
		temp= jcbn[1][0]/jcbn[0][0];
		jcbn[1][1] -= temp*jcbn[0][1];
		rp[1] -= temp*rp[0];
		
		x_new[1] = rp[1]/jcbn[1][1];
		x_new[0] = (rp[0]- x_new[1]*jcbn[0][1])/jcbn[0][0];

    }
   	
	delete r; delete rp; delete x_new; delete jcbn;
	
	return;
}
